/**************************************************************

allparser.h (C-Munipack project)
Readall file parser
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef ALL_PARSER_H
#define ALL_PARSER_H

#include "cmpack_allfile.h"

/***********************      Datatypes      ******************/

/* String buffer */
struct _CmpackString
{
	size_t count;				/**< Number of stored chars */
	size_t capacity;			/**< Number of allocated chars */
	char *buf;					/**< Memory buffer */
};
typedef struct _CmpackString CmpackString;

/* Context structure for catalogue file parser */
struct _CmpackParser
{
	int st;						/**< Automaton state */
	int row;					/**< Current position in file - row index */
	int col;					/**< Current position in file - column index */
	int result;					/**< Result code */
	char lastch;				/**< Previous character */
	CmpackString key;			/**< String buffer for parameter name */
	CmpackString val;			/**< String buffer for parameter value */
	CmpackString jd;			/**< String buffer for julian date */
	CmpackString mag;			/**< String buffer for magnitude */
	CmpackString err;			/**< String buffer for error value */
	CmpackAllFile *file;		/**< Read-all file context */
	int frame, star;			/**< Current frame index, star index */
};
typedef struct _CmpackParser CmpackParser;

/******************   Public functions    *********************/

/* Init parser context */
void all_parser_init(CmpackParser *p, CmpackAllFile *f);

/* Free asociated memory in the context */
void all_parser_clear(CmpackParser *p);

/* Parse buffer (last block must have 'eof' parameter set to nonzero) */
int all_parser_parse(CmpackParser *p, const char *buf, size_t buflen, int eof);
	
#endif
