/**************************************************************

attach.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "comfun.h"
#include "trajd.h"
#include "console.h"
#include "cmpack_common.h"
#include "phot.h"
#include "phfun.h"
#include "attach.h"

int Attach(CmpackPhot *kc, CmpackPhotFrame *frame, CmpackCcdFile *file)
/* Reads lc->image data from fits file into memory buffer */
{
	int res;
	CmpackDateTime dt;
	const char *objname, *filter;

	frame->filter = NULL;
	frame->object = NULL;
	frame->exptime = 0.0;
	frame->ccdtemp = -999.9;
	frame->frsum = frame->fravg = 1;
	frame->ncol = frame->nrow = 0;

    /* Read frame attributes */
	if (cmpack_ccd_get_datetime(file, &dt)!=0)
		return CMPACK_ERR_READ_ERROR;
	frame->jd = cmpack_encodejd(&dt);
	if (frame->jd<=0.0)
		return CMPACK_ERR_INVALID_DATE;

	cmpack_ccd_get_size(file, &frame->ncol, &frame->nrow);
    if (frame->ncol<=0 || frame->ncol>=65536 || frame->nrow<=0 || frame->nrow>=65536) 
		return CMPACK_ERR_INVALID_SIZE;

	filter = cmpack_ccd_get_filter(file);
	if (filter)
		frame->filter = cmpack_strdup(filter);
	objname = cmpack_ccd_get_object(file);
	if (objname)
		frame->object = cmpack_strdup(objname);
	cmpack_ccd_get_exptime(file, &frame->exptime);
	cmpack_ccd_get_ccdtemp(file, &frame->ccdtemp);
	cmpack_ccd_get_frames(file, &frame->fravg);

	/* Compute JD for the center of exposure */
    frame->jd += 0.5*frame->exptime/86400.0;

	/* Read image data */
	frame->ccd = (double*) cmpack_realloc(frame->ccd,frame->nrow*frame->ncol*sizeof(double));
    res = cmpack_ccd_get_image(file, CMPACK_BITPIX_DOUBLE, 0, 0, frame->ccd);
	if (res!=0) {
		cmpack_free(frame->ccd);
		frame->ccd = NULL;
		return res;
	}
	return 0;
}
