/**************************************************************

phot_head.h (C-Munipack project)
Photometry file header
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CAT_HEAD_H
#define CAT_HEAD_H

#include "hash.h"
#include "cmpack_console.h"

/***********************      Datatypes      ******************/

/* Single header item */
typedef struct _CmpackCatHeadItem CmpackCatHeadItem;
	
/* File header */
typedef struct _CmpackCatHeader CmpackCatHeader;

/******************   Public functions    *********************/

/* Init header content */
CmpackCatHeader *cat_head_create();

/* Init header content */
void cat_head_free(CmpackCatHeader *head);

/* Dump header content */
void cat_head_dump(CmpackCatHeader *head, CmpackConsole *ctx);

/* Dump header content */
void cat_head_write(CmpackCatHeader *head, FILE *to);
	
/* Clear header content */
void cat_head_clear(CmpackCatHeader *head);
	
/* Copy header content */
void cat_head_copy(CmpackCatHeader *dsthead, const CmpackCatHeader *srchead);
	
/* Get number of items */
int cat_head_count(CmpackCatHeader *head);

/* Insert new header item or modify existing one */
void cat_head_set(CmpackCatHeader *head, const char *key, const char *val, const char *com);

/* Get header item by its index */
CmpackCatHeadItem *cat_head_get(CmpackCatHeader *head, int index);

/* Get header item by keyword */
CmpackCatHeadItem *cat_head_find(CmpackCatHeader *head, const char *key);
	
/* Delete header item */
void cat_head_delete(CmpackCatHeader *head, CmpackCatHeadItem *item);

/* Get parameter name */
const char *cat_head_item_key(CmpackCatHeadItem *item);

/* Get parameter value */
const char *cat_head_item_val(CmpackCatHeadItem *item);

/* Get comment text */
const char *cat_head_item_com(CmpackCatHeadItem *item);

#endif
