/**************************************************************

phot.h (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef PHOT_H
#define PHOT_H

#include "context.h"
#include "console.h"
#include "cmpack_phot.h"

/* Detected star parameters: */
typedef struct _CmpackPhotStar
{
  int    xmax, ymax;					/* Coordinates of max. brightness pixel */
  double xcen, ycen;           			/* Coordinates of centriod */
  double height;               			/* Max. brightness value */
  double signal;						/* Total sum of brightness */
  double roundness, sharpness; 			/* Shape indices */
  double skymod, skysig, skyskw; 		/* Sky parameters */
  double apmag[MAXAP], magerr[MAXAP];	/* Magnitudes and std. deviations */
  int	 code[MAXAP];					/* Error flags */
  double fwhm;							/* FWHM in pixels */
  struct _CmpackPhotStar *next;     	/* Pointer to next star or NULL */
} CmpackPhotStar;

/* Photometry frame context */
typedef struct _CmpackPhotFrame
{
	double 	jd;						/**< Julian date (center of observation) */
	int ncol, nrow;					/**< Frame width and height in pixels */
	CmpackBorder border;			/**< Border size */
	double *ccd;					/**< Frame image data */
	double exptime;					/**< Exposure duration in seconds */
	char *object;					/**< Object's designation */
	char *filter;					/**< Optical filter name */
	double datalo;					/**< Low good datum in ADU */
	double datahi;					/**< High good datum in ADU */
	int fravg, frsum;				/**< Number of frames averaged, summed */
	double ccdtemp;					/**< CCD temperture in deg. C */
	double thres;					/**< Detection threshold */
	double noise;					/**< Readout noise */
	double phpadu;					/**< ADC gain (photons per ADU) */
	double relerr;					/**< ??? */
	double skymod, skysig;			/**< Sky mean value, standard deviation */
	double maglim, magsq;			/**< ??? */
	double fwhm_med, fwhm_err;		/**< Mean FWHM of objects, standard deviation */
	double aper[MAXAP];				/**< Selected apertures (radii) */
	int apid[MAXAP];				/**< Selected apertures (indices) */
	int naper;						/**< Number of apertures used */
	CmpackPhotStar *first, *last;	/**< List of detected objects */
	int nstar;						/**< Number of stars found */
} CmpackPhotFrame;

/* Photometry configuration context */
struct _CmpackPhot
{
	CmpackContext gc;				/**< Global context */
	CmpackConsole *con;				/**< Console */
	double readns;					/**< Read noise in ADU per frame */
	double gain;					/**< Gain in e-/ADU */
	double datalo;					/**< Low good datum in ADU */
	double datahi;					/**< High good datum in ADU */
	double fwhm;					/**< FWHM od object */
	double thresh;					/**< Threshold in sigmas */
	double shrplo;					/**< Low sharpness cutoff */
	double shrphi;					/**< High sharpness cutoff */
	double rndlo;					/**< Low roundness cutoff */
	double rndhi;					/**< High roundness cutoff */
	CmpackBorder border;			/**< Border size */
	double ap[MAXAP];				/**< Available apertures */
	double is;						/**< Inner sky radius */
	double os;						/**< Outer sky radius */
};

#endif
