/**************************************************************

stddev.h (C-Munipack project)
Standard deviations graph
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef LIST_STDDEV_H
#define LIST_STDDEV_H

#include "cmpack_list.h"
#include "cmpack_phtfile.h"

/* Measurements */
struct _CmpackStddevMag
{
	int id;							/**< Aperture id */
	double mag;						/**< Magnitude */
	struct _CmpackStddevMag *next;	/**< Next measurement */
};
typedef struct _CmpackStddevMag CmpackStddevMag;

/* Star info */
struct _CmpackStddevStar
{
	int	id;							/**< Star identifier */
    CmpackStddevMag *first, *last;	/**< Measurements */
    struct _CmpackStddevStar *next;	/**< Next star */
};
typedef struct _CmpackStddevStar CmpackStddevStar;

/* Frame info */
struct _CmpackStddevFrame
{
    CmpackStddevStar *first, *last;	/**< Star data */
};
typedef struct _CmpackStddevFrame CmpackStddevFrame;

/* Table of frames */
struct _CmpackStddevFrameList
{
	CmpackStddevFrame *list;			/**< List of frames */
	int count;						/**< Number of items */
	int capacity;					/**< Number of allocated records */
};
typedef struct _CmpackStddevFrameList CmpackStddevFrameList;

/* Aperture info */
struct _CmpackStddevAperture
{
	int id;							/**< Aperture id */
	double radius;					/**< Aperture radius in pixels */
};
typedef struct _CmpackStddevAperture CmpackStddevAperture;

/* Table of apertures */
typedef struct _CmpackStddevAperList
{
	CmpackStddevAperture *list;			/**< List of apertures */
	int count;						/**< Number of items */
	int capacity;					/**< Number of allocated records */
} CmpackStddevAperList;

/* Initialize variables */
int list_dev_init(CmpackLister *lc);
	
/* Process single file */
int list_dev(CmpackLister *lc, CmpackPhtFile *pht, int frame);
	
/* Finish reading */
int list_dev_close(CmpackLister *lc);

#endif
