/**************************************************************

addfolder_dlg.cpp (C-Munipack project)
The 'Add folder' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "addfolder_dlg.h"
#include "addfilter_dlg.h"
#include "project.h"
#include "configuration.h"
#include "utils.h"
#include "main.h"

CAddFolderDlg::CAddFolderDlg(GtkWindow *pParent)
{
	GtkWidget *frame, *vbox;

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Add folder", pParent, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
		GTK_STOCK_CLOSE, GTK_RESPONSE_REJECT, GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("addfolder");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Options
	frame = gtk_frame_new("Search options");
	vbox = gtk_vbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	m_SubDirs = gtk_check_button_new_with_label("Include subdirectories");
	gtk_box_pack_start(GTK_BOX(vbox), m_SubDirs, TRUE, TRUE, 0);
	gtk_widget_show_all(frame);
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(m_pDlg), frame);

	// Signals
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
}

CAddFolderDlg::~CAddFolderDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CAddFolderDlg::Execute()
{
	// Go to last folder
	char *path = CConfig::GetStr("AddFolder", "Folder", NULL);
	if (path) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), path);
		g_free(path);
	}

	// Restore last state
	bool subdirs = CConfig::GetBool("AddFolder", "SubDirs", true);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_SubDirs), subdirs);

	gtk_dialog_run(GTK_DIALOG(m_pDlg));

	Global.pProject->Save();
}

void CAddFolderDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAddFolderDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CAddFolderDlg::OnCloseQuery(gint response_id)
{
	bool	subdirs;
	gchar	*path, *dir;
	CAddFilterDlg *pFilterDlg;
	
	if (response_id == GTK_RESPONSE_ACCEPT) {
		// Save last state
		path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		CConfig::SetStr("AddFolder", "Folder", (path ? path : ""));
		subdirs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_SubDirs));
		CConfig::SetBool("AddFolder", "SubDirs", subdirs);
		dir = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		pFilterDlg = new CAddFilterDlg(GTK_WINDOW(m_pDlg));
		pFilterDlg->Execute(dir, subdirs);
		delete pFilterDlg;
		g_free(dir);
		g_free(path);
		return false;
	}
	return true;
}

