/**************************************************************

convert_dlg.cpp (C-Munipack project)
The 'Convert files' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "project.h"
#include "configuration.h"
#include "convert_dlg.h"
#include "progress_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"

//-------------------------   MAIN WINDOW   --------------------------------

CConvertDlg::CConvertDlg(GtkWindow *pParent):m_AllFiles(false)
{
	GtkWidget *frame, *vbox;
	GSList *group;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Fetch/convert files", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("convert");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Frame
	frame = gtk_frame_new("Process");
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), frame);
	gtk_widget_set_size_request(frame, 300, -1);
	gtk_widget_show(frame);

	// Frame layout
	vbox = gtk_vbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_widget_show(vbox);

	// Options
	m_AllBtn = gtk_radio_button_new_with_label(NULL, "all files in current project");
	gtk_box_pack_start(GTK_BOX(vbox), m_AllBtn, TRUE, TRUE, 0);
	gtk_widget_show(m_AllBtn);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_AllBtn));

	m_SelBtn = gtk_radio_button_new_with_label(group, "selected files only");
	gtk_box_pack_start(GTK_BOX(vbox), m_SelBtn, TRUE, TRUE, 0);
	gtk_widget_show(m_SelBtn);

	// Default state
	GtkTreeSelection *pSel = Global.pMainWnd->GetSelection();
	if (gtk_tree_selection_count_selected_rows(pSel)>1) 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_SelBtn), true);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_AllBtn), true);
}

CConvertDlg::~CConvertDlg()
{
	gtk_widget_destroy(m_pDlg);
}

gboolean CConvertDlg::foreach_all_files(GtkTreeModel *model, GtkTreePath *path, 
										GtkTreeIter *iter, gpointer userdata)
{
	GList **list = ((GList**)userdata);
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
	return FALSE;
}

void CConvertDlg::foreach_sel_files(GtkTreeModel *model, GtkTreePath *path, 
							   GtkTreeIter *iter, gpointer userdata)
{
	GList **list = ((GList**)userdata);
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
}

void CConvertDlg::Execute()
{
	int res;
	char msg[256];
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return;

	gtk_widget_hide(m_pDlg);
	m_FileList = NULL;
	m_AllFiles = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_AllBtn));
	if (m_AllFiles) {
		// All files
		GtkTreeModel *pList = Global.pProject->FileList();
		if (gtk_tree_model_iter_n_children(pList, NULL)>0) 
			gtk_tree_model_foreach(pList, foreach_all_files, &m_FileList);
		else
			ShowError(pMainWnd, "There are no files in the project.");
	} else {
		// Selected files
		GtkTreeSelection *pSel = Global.pMainWnd->GetSelection();
		if (gtk_tree_selection_count_selected_rows(pSel)>0) 
			gtk_tree_selection_selected_foreach(pSel, foreach_sel_files, &m_FileList);
		else
			ShowError(pMainWnd, "There are no selected files.");
	}
	if (m_FileList) {
		if (m_AllFiles) {
			Global.pProject->ClearTempFiles();
			Global.pProject->ClearCorrections();
			Global.pProject->ClearReference();
			Global.pProject->ClearSelection();
			Global.pProject->ClearObject();
			Global.pProject->ClearObserver();
			Global.pProject->SetAperture(1);
		}
		CProgressDlg pDlg(Global.pMainWnd->GetHandle(), "Converting files...");
		pDlg.SetMinMax(0, g_list_length(m_FileList));
		res = pDlg.Execute(ExecuteProc, this);
		if (res>0) {
			char *msg = cmpack_formaterror(res);
			ShowError(pMainWnd, msg);
			cmpack_free(msg);
		} else
		if (m_OutFiles==0) {
			ShowError(pMainWnd, "No file was successfully processed.");
		} else {
			if (m_OutFiles!=m_InFiles) {
				sprintf(msg, "%d file(s) was successfully processed, %d file(s) failed.", 
					m_OutFiles, m_InFiles-m_OutFiles);
				ShowWarning(pMainWnd, msg);
			} else {
				sprintf(msg, "All %d file(s) was successfully processed.", m_OutFiles);
				ShowInformation(pMainWnd, msg);
			}
		}
		g_list_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
		g_list_free(m_FileList);
		m_FileList = NULL;
	}
	Global.pProject->Save();
	Global.pProject->UpdateAllIcons();
}

int CConvertDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CConvertDlg*)userdata)->ConvertFiles(sender);
}

int CConvertDlg::ConvertFiles(CProgressDlg *sender)
{
	int res, frameid;
	char *fpath, *fbase, msg[128];
	GtkTreePath *path;
	CConvertProc konv;

	m_InFiles = m_OutFiles = 0;
	sender->Print("------ Fetch/convert files ------");

	res = konv.Init(sender);
	if (res==0) {
		for (GList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			frameid = Global.pProject->GetFrameID(path);
			fpath = Global.pProject->GetSourceFile(path);
			fbase = g_filename_display_basename(fpath);
			sender->SetFileName(fbase);
			g_free(fbase);
			g_free(fpath);
			sprintf(msg, "Frame #%d:", frameid);
			sender->Print(msg);
			sender->SetProgress(m_InFiles++);
			if (konv.Execute(path)==0)
				m_OutFiles++;
			gtk_tree_path_free(path);
			if (sender->Cancelled()) 
				break;
		}
		if (sender->Cancelled())
			sprintf(msg, "Cancelled at the user's request");
		else
			sprintf(msg, "====== %d succeeded, %d failed ======", m_OutFiles, m_InFiles-m_OutFiles);
		sender->Print(msg);
	} else {
		char *aux = cmpack_formaterror(res);
		sender->Print(msg);
		cmpack_free(aux);
	}
	return res;
}
