/**************************************************************

graph_data.h (C-Munipack project)
An object which holds graph data
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_GRAPH_DATA_H
#define MUNIWIN_GRAPH_DATA_H

#include "callback.h"

class CGraphData
{
public:
	// Constructor
	CGraphData();
	CGraphData(int channels, int capacity = 0);

	// Destructor
	virtual ~CGraphData();

	// Set reference to the graph
	void Connect(class CGraph *pGraph);
	void Disconnect(class CGraph *pGraph);

	// Lock/unlock invalidates
	void LockRedraw(void);
	void UnlockRedraw(void);

	// Clear data and set number of channels and initial capacity
	void Init(int channels, int capacity = 0);

	// Clear all data
	void Clear();

	// Add new data to the graph
	void Append(double x, const double *values);

	// Get number of measurements
	int Count(void) const
	{ return m_Count; }

	// Set X value
	void SetX(int index, double x);

	// Set values
	void SetValues(int index, const double *values);

	// Get X value 
	double GetX(int index) const
	{ return m_Data[index*m_RecSize]; }

	// Get values
	const double *GetValues(int index) const
	{ return m_Data+(index*m_RecSize+1); }

	// Get values
	const double GetY(int index, int channel = 0) const
	{ return m_Data[index*m_RecSize+(channel+1)]; }

protected:
	int			m_Channels, m_RecSize;
	int			m_Count, m_Capacity;
	double		*m_Data;

	int			m_GraphCount, m_GraphCapacity;
	CGraph		**m_GraphList;

	bool		m_DataChanged;
	int			m_RedrawLockCount;

	// Invalidate graph
	void Invalidate();
};

#endif
