/**************************************************************

varfind_dlg.cpp (C-Munipack project)
The 'Find variables' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "project.h"
#include "configuration.h"
#include "graph.h"
#include "progress_dlg.h"
#include "varfind_dlg.h"
#include "main.h"
#include "utils.h"
#include "chartexport_dlg.h"

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CVarFindDlg::CVarFindDlg(GtkWindow *pParent):m_MagDevData(1), m_LightCurveData(2)
{
	int width, height;
	GtkWidget *frame, *table, *hbox, *vbox, *separator, *box2;
	GSList *group;

	m_VarID = m_CompID = -1;
	m_SelectMode = SELECT_VAR;
	m_Updating = false;
	m_LastStarIndex = m_LastPtIndex = -1;
	m_DispMode = (tDispMode)CConfig::GetInt("VarFindDlg", "Mode", DISP_FULL);

	// Create table context
	m_AllFile = cmpack_all_init();
	m_MagDevTable = cmpack_tab_init(CMPACK_TABLE_MUNIFIND);
	m_LightCurveTable = cmpack_tab_init(CMPACK_TABLE_LIGHTCURVE);

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Find variables", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("varfind");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Set window size
	GdkScreen *scr = gtk_window_get_screen(GTK_WINDOW(m_pDlg));
	if (scr) {
		width = gdk_screen_get_width(scr) - 64;
		height = gdk_screen_get_height(scr) - 120;
		gtk_window_set_default_size(GTK_WINDOW(m_pDlg), width, height);
		gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
	}

	// Dialog layout
	hbox = gtk_hbox_new(FALSE, 8);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), hbox);

	// Table layout
	table = gtk_table_new(2, 2, true);
	gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
	gtk_widget_set_size_request(table, 640, 480);

	// Varfind-curve graph
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_MagDevGraph.SetGraphPadding(2, 2, 32, 2);
	m_MagDevGraph.EnableTracking(true);
	m_MagDevGraph.RegisterCallback(GraphCallback, this);
	m_MagDevGraph.SetData(&m_MagDevData);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_MagDevGraph.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 1, 0, 1);

	// Chart
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_Chart.ShowBadStars(false);
	m_Chart.ShowNonMatchedStars(false);
	m_Chart.EnableTracking(true);
	m_Chart.RegisterCallback(ChartCallback, this);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 1, 2, 0, 1);

	// Light-curve graph
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_LightCurveGraph.EnableTracking(false);
	m_LightCurveGraph.SetData(&m_LightCurveData);
	m_MagDevGraph.SetGraphPadding(2, 2, 48, 2);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_LightCurveGraph.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 2, 1, 2);

	// Selection mode
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
	m_VarBtn = gtk_toggle_button_new_with_label("Select variable star");
	gtk_box_pack_start(GTK_BOX(vbox), m_VarBtn, FALSE, TRUE, 0);
	m_CompBtn = gtk_toggle_button_new_with_label("Select comparison star");
	gtk_box_pack_start(GTK_BOX(vbox), m_CompBtn, FALSE, TRUE, 0);

	// Display mode
	frame = gtk_frame_new("Show");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
	box2 = gtk_vbox_new(TRUE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(box2), 4);
	gtk_container_add(GTK_CONTAINER(frame), box2);
	m_ShowImage = gtk_radio_button_new_with_label(NULL, "Image only");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowImage, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowChart = gtk_radio_button_new_with_label(group, "Chart only");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowChart, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowFull = gtk_radio_button_new_with_label(group, "Image + chart");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowFull, TRUE, TRUE, 0);

	// Separator
	separator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, TRUE, 0);

	// Save buttons
	m_SaveMagDev = gtk_button_new_with_label("Save mag-dev data");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveMagDev, FALSE, TRUE, 0);
	m_SaveChart = gtk_button_new_with_label("Save chart");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveChart, FALSE, TRUE, 0);
	m_SaveCurve = gtk_button_new_with_label("Save light curve");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveCurve, FALSE, TRUE, 0);
	m_Export = gtk_button_new_with_label("Save read-all data");
	gtk_box_pack_start(GTK_BOX(vbox), m_Export, FALSE, TRUE, 0);

	SetDisplayMode(m_DispMode);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowImage), m_DispMode==DISP_IMAGE);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowChart), m_DispMode==DISP_CHART);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowFull), m_DispMode==DISP_FULL);

	gtk_widget_show_all(hbox);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_VarBtn), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_CompBtn), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_SaveMagDev), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_SaveChart), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_SaveCurve), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_Export), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_ShowImage), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_ShowChart), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_ShowFull), "toggled", G_CALLBACK(toggled), this);
}

CVarFindDlg::~CVarFindDlg()
{
	gtk_widget_destroy(m_pDlg);
	cmpack_unref(m_MagDevTable);
	cmpack_unref(m_LightCurveTable);
	cmpack_unref(m_AllFile);
}

void CVarFindDlg::Execute()
{
	int i, res, refid;
	char *tpath, *ppath;
	const char *cat_file, *tmp_file;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();

	m_FileList = NULL;
	m_Chart.Clear();
	m_MagDevData.Clear();
	m_LightCurveData.Clear();

	// Check reference file
	Global.pProject->GetReference(&refid, &cat_file);
	if (refid>0) {
		// Load reference frame
		ppath = Global.pProject->GetPhotFile(refid);
		m_Chart.LoadChart(ppath, true);
		tpath = Global.pProject->GetImageFile(refid);
		m_Chart.LoadImage(tpath);
		g_free(ppath);
		g_free(tpath);
	} else if (cat_file) {
		// Load catalogue file
		tmp_file = Global.pProject->GetTempCatFile();
		m_Chart.LoadChart(tmp_file, true);
		ppath = SetFileExtension(tmp_file, "fts");
		m_Chart.LoadImage(ppath);
		g_free(ppath);
	} else {
		ShowError(pMainWnd, "No reference file.");
		return;
	}

	// Check aperture
	m_Aperture = Global.pProject->Aperture();
	m_Chart.SetAperture(m_Aperture);

	// Initial selection
	m_CompID = -1;
	const CSelection *sel = Global.pProject->Selection();
	i = sel->FindStar(ST_COMPARISON, 0);
	if (i>=0) 
		m_CompID = sel->GetId(i);
	i = sel->FindStar(ST_VARIABLE, 0);
	if (i>=0)
		m_VarID = sel->GetId(i);

	// Make list of input files
	gtk_tree_model_foreach(Global.pProject->FileList(), (GtkTreeModelForeachFunc)foreach_all_files, &m_FileList);
	if (!m_FileList) {
		ShowError(pMainWnd, "There are no valid frames in the list.");
		return;
	}

	m_Cancelled = false;
	CProgressDlg pDlg(Global.pMainWnd->GetHandle(), "Processing files...");
	res = pDlg.Execute(ExecuteProc, this);

	// Free file list
	g_slist_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
	g_slist_free(m_FileList);
	m_FileList = NULL;

	if (m_Cancelled)
		return;
	if (res!=0) {
		char *msg = cmpack_formaterror(res);
		ShowError(pMainWnd, msg);
		cmpack_free(msg);
		return;
	}

	RebuildData(pMainWnd);
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
}

void CVarFindDlg::RebuildData(GtkWindow *pParent)
{
	int		res, row_count, i;
	double	mag, data[2];
	CmpackMuniFind *mfind;

	cmpack_tab_clear(m_MagDevTable);
	cmpack_tab_clear(m_LightCurveTable);
	m_MagDevData.Clear();
	m_LightCurveData.Clear();

	mfind = cmpack_mfind_init();
	cmpack_mfind_set_threshold(mfind, CConfig::GetDbl("MuniFind", "Threshold", 60.0));
	if (m_CompID>=0)
		res = cmpack_mfind(mfind, m_AllFile, m_MagDevTable, m_CompID);
	else
		res = cmpack_mfind(mfind, m_AllFile, m_MagDevTable, CMPACK_MFIND_AUTO);
	if (res==0) 
		m_CompID = cmpack_mfind_get_comparison(mfind);
	cmpack_unref(mfind);

	if (res!=0) {
		char *msg = cmpack_formaterror(res);
		ShowError(pParent, msg);
		cmpack_free(msg);
	}
	
	// Load mag-dev data
	m_MagDevData.LockRedraw();
	m_MagDevData.Clear();
	row_count = cmpack_tab_nrows(m_MagDevTable);
	m_MagDevData.Init(2, row_count);
	m_MinMag = m_MaxMag = m_MaxDev = 0.0;
	for (i=0; i<row_count; i++) {
		cmpack_tab_gtdd(m_MagDevTable, i, 1, &mag);
		cmpack_tab_gtdd(m_MagDevTable, i, 2, &data[0]);
		cmpack_tab_gtdd(m_MagDevTable, i, 0, &data[1]);
		if (mag>-99 && mag<99) {
			if (mag < m_MinMag)
				m_MinMag = mag;
			if (mag > m_MaxMag)
				m_MaxMag = mag;
			if (data[0] > m_MaxDev)
				m_MaxDev = data[0];
			m_MagDevData.Append(mag, data);
		}
	}
	m_MagDevData.UnlockRedraw();
	m_MagDevGraph.ClearColumns();
	m_MagDevGraph.SetXParams("mag", m_MinMag, m_MaxMag, 0.5, 0, 1);
	m_MagDevGraph.SetYParams("dev", 0, m_MaxDev, 0.1, 0, 1);
	m_MagDevGraph.AddColumn(0, -1, 0);
	m_MagDevGraph.AutoZoom(true, true);

	m_LastStarIndex = m_LastPtIndex = -1;
	UpdateSelection();
}

int CVarFindDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CVarFindDlg*)userdata)->ProcessFiles(sender);
}

int CVarFindDlg::ProcessFiles(CProgressDlg *sender)
{
	int res, frameid, infiles;
	char *temp, *file, *f, msg[128];
	GtkTreePath *path;
	CmpackPhtFile *pht;

	sender->SetMinMax(0, g_slist_length(m_FileList));

	CmpackConsole *con = cmpack_con_init_cb(CProgressDlg::OutputProc, sender);
	if (CConfig::GetBool("Environment", "Debug"))
		cmpack_con_set_level(con, CMPACK_LEVEL_DEBUG);

	CmpackReadAll *readall = cmpack_readall_init();
	cmpack_readall_set_console(readall, con);
	cmpack_readall_set_aperture(readall, m_Aperture);

	// Make listing context
	res = cmpack_readall_open(readall, m_AllFile);
	if (res==0) {
		// Process files
		infiles = 0;
		for (GSList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			frameid = Global.pProject->GetFrameID(path);
			temp = Global.pProject->GetPhotFileName(path);
			sender->SetFileName(temp);
			g_free(temp);
			sprintf(msg, "Frame #%d:", frameid);
			sender->Print(msg);
			file = Global.pProject->GetPhotFile(path);
			f = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
			res = cmpack_pht_load(&pht, f, CMPACK_LOAD_DEFAULT);
			if (res==0) {
				res = cmpack_readall_read(readall, pht);
				cmpack_unref(pht);
			}
			g_free(f);
			g_free(file);
			if (res!=0)
				break;
			if (sender->Cancelled()) {
				m_Cancelled = true;
				break;
			}
			sender->SetProgress(infiles++);
		}
		// Free allocated memory
		cmpack_readall_close(readall);
	}
	cmpack_unref(readall);
	cmpack_unref(con);
	return res;
}

gboolean CVarFindDlg::foreach_all_files(GtkTreeModel *model, GtkTreePath *path, 
										GtkTreeIter *iter, GSList **list)
{
	if (Global.pProject->GetState(iter) & CFILE_MATCHING)
		*list = g_slist_append(*list, gtk_tree_row_reference_new(model, path));
	return FALSE;
}

void CVarFindDlg::toggled(GtkToggleButton *pBtn, CVarFindDlg *pDlg)
{
	pDlg->OnToggled(pBtn);
}

void CVarFindDlg::OnToggled(GtkToggleButton *pBtn)
{
	if (gtk_toggle_button_get_active(pBtn)!=0) {
		if (pBtn==GTK_TOGGLE_BUTTON(m_VarBtn))
			SetMode(SELECT_VAR);
		else if (pBtn==GTK_TOGGLE_BUTTON(m_CompBtn))
			SetMode(SELECT_COMP);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowImage)) 
			SetDisplayMode(DISP_IMAGE);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowChart))
			SetDisplayMode(DISP_CHART);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowFull))
			SetDisplayMode(DISP_FULL);
	}
}

void CVarFindDlg::clicked(GtkButton *pBtn, CVarFindDlg *pDlg)
{
	pDlg->OnClicked(pBtn);
}

void CVarFindDlg::OnClicked(GtkButton *pBtn)
{
	if (pBtn==GTK_BUTTON(m_SaveMagDev))
		SaveMagDev();
	else if (pBtn==GTK_BUTTON(m_SaveChart))
		SaveChart();
	else if (pBtn==GTK_BUTTON(m_SaveCurve))
		SaveCurve();
	else if (pBtn==GTK_BUTTON(m_Export))
		Export();
}

void CVarFindDlg::response_dialog(GtkDialog *pDlg, gint response_id, CVarFindDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CVarFindDlg::OnResponseDialog(gint response_id)
{
	if (response_id == GTK_RESPONSE_ACCEPT) {
		Global.pProject->SetSelection(m_Chart.GetSelection());
		Global.pProject->SetVarFindSelection(m_Chart.GetSelection());
	}
	return true;
}

void CVarFindDlg::SetMode(tSelectMode mode)
{
	if (!m_Updating) {
		m_Updating = true;
		m_SelectMode = mode;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_VarBtn), mode==SELECT_VAR);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CompBtn), mode==SELECT_COMP);
		m_Updating = false;
	}
}

void CVarFindDlg::ChartCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CVarFindDlg *pMe = (CVarFindDlg*)cb_data;
		
	switch (message)
	{
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnChartMouseMove();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnChartLButtonClick((GdkEventButton*)lparam);
		break;
	}
}

void CVarFindDlg::OnChartMouseMove()
{
	int index, x, y;

	if (m_Chart.GetCursorPos(&x, &y)) 
		index = m_Chart.FindStarByPos(x, y, false);
	else
		index = -1;
	if (index>=0) {
		if (m_LastStarIndex!=index) {
			m_LastStarIndex = index;
			m_Chart.SetFocusedStar(index);
		}
	} else {
		if (m_LastStarIndex>=0) {
			m_LastStarIndex = -1;
			m_Chart.SetFocusedStar(-1);
		}
	}
}

void CVarFindDlg::OnChartLButtonClick(const GdkEventButton *event)
{
	CChartBase::tStarInfo star;

	if (m_Chart.GetStar(m_Chart.FocusedStar(), &star)) {
		if (m_SelectMode == SELECT_VAR) {
			if (star.id!=m_VarID && star.id!=m_CompID) {
				m_VarID = star.id;
				UpdateSelection();
			}
		} else {
			if (star.id!=m_CompID) {
				m_CompID = star.id;
				if (m_CompID == m_VarID)
					m_VarID = -1;
				RebuildData(GTK_WINDOW(m_pDlg));
				UpdateSelection();
			}
		}
	}
}

void CVarFindDlg::GraphCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CVarFindDlg *pMe = (CVarFindDlg*)cb_data;
		
	switch (message)
	{
	case CGraph::CB_MOUSE_MOVE:
		pMe->OnGraphMouseMove();
		break;
	case CGraph::CB_LEFT_CLICK:
		pMe->OnGraphLButtonClick((GdkEventButton*)lparam);
		break;
	}
}

void CVarFindDlg::OnGraphMouseMove(void)
{
	int index;
	double x, y;

	if (m_MagDevGraph.GetMousePos(&x, &y)) 
		index = m_MagDevGraph.FindByPos(x, y, 0);
	else
		index = -1;
	if (index>=0) {
		if (m_LastPtIndex!=index) {
			m_LastPtIndex = index;
			m_MagDevGraph.SetFocusedItem(index, 0);
		}
	} else {
		if (m_LastPtIndex>=0) {
			m_LastPtIndex = -1;
			m_MagDevGraph.SetFocusedItem(-1, -1);
		}
	}
}

void CVarFindDlg::OnGraphLButtonClick(const GdkEventButton *event)
{
	if (m_LastPtIndex>=0) {
		int starid = (int)m_MagDevData.GetY(m_LastPtIndex, 1);
		if (m_SelectMode == SELECT_VAR) {
			if (starid!=m_VarID && starid!=m_CompID) {
				m_VarID = starid;
				UpdateSelection();
			}
		} else {
			if (starid!=m_CompID) {
				m_CompID = starid;
				if (m_CompID == m_VarID)
					m_VarID = -1;
				RebuildData(GTK_WINDOW(m_pDlg));
				UpdateSelection();
			}
		}
	}
}


void CVarFindDlg::UpdateSelection(void)
{
	int i, count, row_count;
	double jd, yvals[2];

	m_Chart.GetSelection()->Clear();
	if (m_VarID>=0)
		m_Chart.GetSelection()->Select(m_VarID, ST_VARIABLE);
	if (m_CompID>=0)
		m_Chart.GetSelection()->Select(m_CompID, ST_COMPARISON);
	
	m_MagDevGraph.DeleteAllTags();
	count = m_MagDevData.Count();
	for (i=0; i<count; i++) {
		if ((int)m_MagDevData.GetY(i, 1)==m_VarID) {
			m_MagDevGraph.SetTag(0, i, "var", RGB(255,0,0));
			break;
		}
	}

	/* Update light-curve */
	m_LightCurveData.LockRedraw();
	m_LightCurveData.Clear();
	m_JDMin = m_JDMax = 0.0;
	cmpack_tab_clear(m_LightCurveTable);
	if (m_VarID>=0 && m_CompID>=0) {
		cmpack_all_lightcurve(m_AllFile, m_VarID, m_CompID, m_LightCurveTable);
		row_count = cmpack_tab_nrows(m_LightCurveTable);
		if (row_count>0) {
			m_LightCurveData.Init(2, row_count);
			for (i=0; i<row_count; i++) {
				cmpack_tab_gtdd(m_LightCurveTable, i, 0, &jd);
				cmpack_tab_gtdd(m_LightCurveTable, i, 1, &yvals[0]);
				cmpack_tab_gtdd(m_LightCurveTable, i, 2, &yvals[1]);
				m_LightCurveData.Append(jd, yvals);
				if (i==0) {
					m_JDMin = m_JDMax = jd;
				} else {
					if (jd < m_JDMin)
						m_JDMin = jd;
					if (jd > m_JDMax)
						m_JDMax = jd;
				}
			}
			m_LightCurveGraph.SetXParams("JD", m_JDMin, m_JDMax, 1.0/144.0, 0, 2);
			m_LightCurveGraph.ClearColumns();
			m_LightCurveGraph.AddColumn(0, 1, 0);
			m_LightCurveGraph.SetYParams("V-C", -99, 99, 0.5, CGraph::REVERSE_SCALE, 2);
			m_LightCurveGraph.AutoZoom(true, true);
		}
	}
	m_LightCurveData.UnlockRedraw();
}

void CVarFindDlg::SaveMagDev(void)
{
	char *filename, *folder, *fpath, *f;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save varfind data to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("VarFind", "MagDevName", "output.txt");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("VarFind", "MagDevName", filename);
		g_free(filename);
		f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_tab_save(m_MagDevTable, f);
		g_free(f);
		g_free(fpath);
	}
	gtk_widget_destroy(pSaveDlg);
}

void CVarFindDlg::SaveCurve(void)
{
	char *filename, *folder, *fpath, *f;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save light-curve to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("VarFind", "LightCurveName", "output.txt");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("VarFind", "LightCurveName", filename);
		g_free(filename);
		f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_tab_save(m_LightCurveTable, f);
		g_free(f);
		g_free(fpath);
	}
	gtk_widget_destroy(pSaveDlg);
}

void CVarFindDlg::SaveChart()
{
	CChartExportDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(&m_Chart);
}

void CVarFindDlg::SetDisplayMode(tDispMode mode)
{
	m_DispMode = mode;
	CConfig::SetInt("VarFindDlg", "Mode", m_DispMode);

	m_Chart.ShowImage(m_DispMode!=DISP_CHART);
	m_Chart.ShowChart(m_DispMode!=DISP_IMAGE);
}

void CVarFindDlg::Export(void)
{
	int refid;
	const char *cat_file, *tmp_file;
	char *filename, *folder, *fpath, *f1, *dst_path, *src_path;
	gchar *contents;
	gsize length;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Export data to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("VarFind", "ExportName", "munifind.dat");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("VarFind", "ExportName", filename);
		g_free(filename);
		// Save readall data
		f1 = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_all_save(m_AllFile, f1);
		g_free(f1);
		Global.pProject->GetReference(&refid, &cat_file);
		if (refid>0) {
			// Save reference CCD image
			dst_path = SetFileExtension(fpath, "fts");
			src_path = Global.pProject->GetImageFile(refid);
			if (g_file_get_contents(src_path, &contents, &length, NULL)) {
				g_file_set_contents(dst_path, contents, length, NULL);
				g_free(contents);
			}
			g_free(src_path);
			g_free(dst_path);
			// Save reference photometry file
			dst_path = SetFileExtension(fpath, "pht");
			src_path = Global.pProject->GetPhotFile(refid);
			if (g_file_get_contents(src_path, &contents, &length, NULL)) {
				g_file_set_contents(dst_path, contents, length, NULL);
				g_free(contents);
			}
			g_free(src_path);
			g_free(dst_path);
		} else if (cat_file) {
			// Copy catalogue file
			tmp_file = Global.pProject->GetTempCatFile();
			dst_path = SetFileExtension(fpath, "xml");
			if (g_file_get_contents(tmp_file, &contents, &length, NULL)) {
				g_file_set_contents(dst_path, contents, length, NULL);
				g_free(contents);
			}
			g_free(dst_path);
			// Copy image file
			src_path = SetFileExtension(tmp_file, "fts");
			dst_path = SetFileExtension(fpath, "fts");
			if (g_file_get_contents(src_path, &contents, &length, NULL)) {
				g_file_set_contents(dst_path, contents, length, NULL);
				g_free(contents);
			}
			g_free(dst_path);
			g_free(src_path);
		}
		g_free(fpath);
	}
	gtk_widget_destroy(pSaveDlg);
}
